@@ -8,6 +8,7 @@ import (
 	"io/ioutil"
 	"math"
 	"net/http"
+	"net/url"
 	"os"
 	"regexp"
 	"strconv"
@@ -24,6 +25,8 @@ import (
 
 var Version string = "0.0.1"
 
+var db *sql.DB = nil
+
 var (
 	listenAddress = flag.String(
 		"web.listen-address", ":9187",
@@ -92,12 +95,6 @@ const (
 	DURATION     ColumnUsage = iota // This column should be interpreted as a text duration (and converted to milliseconds)
 )
 
-// Special case matric mappings
-const (
-	// Which metric mapping should be acquired using "SHOW" queries
-	SHOW_METRIC = "pg_runtime_variables"
-)
-
 // Regex used to get the "short-version" from the postgres version field.
 var versionRegex = regexp.MustCompile(`^\w+ (\d+\.\d+\.\d+)`)
 var lowestSupportedVersion = semver.MustParse("9.1.0")
@@ -144,24 +141,6 @@ type MetricMap struct {
 	conversion func(interface{}) (float64, bool) // Conversion function to turn PG result into float64
 }
 
-// Metric descriptors for dynamically created metrics.
-var variableMaps = map[string]map[string]ColumnMapping{
-	"pg_runtime_variable": map[string]ColumnMapping{
-		"max_connections":                {GAUGE, "Sets the maximum number of concurrent connections.", nil, nil},
-		"max_files_per_process":          {GAUGE, "Sets the maximum number of simultaneously open files for each server process.", nil, nil},
-		"max_function_args":              {GAUGE, "Shows the maximum number of function arguments.", nil, nil},
-		"max_identifier_length":          {GAUGE, "Shows the maximum identifier length.", nil, nil},
-		"max_index_keys":                 {GAUGE, "Shows the maximum number of index keys.", nil, nil},
-		"max_locks_per_transaction":      {GAUGE, "Sets the maximum number of locks per transaction.", nil, nil},
-		"max_pred_locks_per_transaction": {GAUGE, "Sets the maximum number of predicate locks per transaction.", nil, nil},
-		"max_prepared_transactions":      {GAUGE, "Sets the maximum number of simultaneously prepared transactions.", nil, nil},
-		//"max_stack_depth" : { GAUGE, "Sets the maximum number of concurrent connections.", nil }, // No dehumanize support yet
-		"max_standby_archive_delay":   {DURATION, "Sets the maximum delay before canceling queries when a hot standby server is processing archived WAL data.", nil, nil},
-		"max_standby_streaming_delay": {DURATION, "Sets the maximum delay before canceling queries when a hot standby server is processing streamed WAL data.", nil, nil},
-		"max_wal_senders":             {GAUGE, "Sets the maximum number of simultaneously running WAL sender processes.", nil, nil},
-	},
-}
-
 // TODO: revisit this with the semver system
 func dumpMaps() {
 	for name, cmap := range metricMaps {
@@ -182,7 +161,7 @@ func dumpMaps() {
 }
 
 var metricMaps = map[string]map[string]ColumnMapping{
-	"pg_stat_bgwriter": map[string]ColumnMapping{
+	"pg_stat_bgwriter": {
 		"checkpoints_timed":     {COUNTER, "Number of scheduled checkpoints that have been performed", nil, nil},
 		"checkpoints_req":       {COUNTER, "Number of requested checkpoints that have been performed", nil, nil},
 		"checkpoint_write_time": {COUNTER, "Total amount of time that has been spent in the portion of checkpoint processing where files are written to disk, in milliseconds", nil, nil},
@@ -195,7 +174,7 @@ var metricMaps = map[string]map[string]ColumnMapping{
 		"buffers_alloc":         {COUNTER, "Number of buffers allocated", nil, nil},
 		"stats_reset":           {COUNTER, "Time at which these statistics were last reset", nil, nil},
 	},
-	"pg_stat_database": map[string]ColumnMapping{
+	"pg_stat_database": {
 		"datid":          {LABEL, "OID of a database", nil, nil},
 		"datname":        {LABEL, "Name of this database", nil, nil},
 		"numbackends":    {GAUGE, "Number of backends currently connected to this database. This is the only column in this view that returns a value reflecting current state; all other columns return the accumulated values since the last reset.", nil, nil},
@@ -216,7 +195,7 @@ var metricMaps = map[string]map[string]ColumnMapping{
 		"blk_write_time": {COUNTER, "Time spent writing data file blocks by backends in this database, in milliseconds", nil, nil},
 		"stats_reset":    {COUNTER, "Time at which these statistics were last reset", nil, nil},
 	},
-	"pg_stat_database_conflicts": map[string]ColumnMapping{
+	"pg_stat_database_conflicts": {
 		"datid":            {LABEL, "OID of a database", nil, nil},
 		"datname":          {LABEL, "Name of this database", nil, nil},
 		"confl_tablespace": {COUNTER, "Number of queries in this database that have been canceled due to dropped tablespaces", nil, nil},
@@ -225,12 +204,12 @@ var metricMaps = map[string]map[string]ColumnMapping{
 		"confl_bufferpin":  {COUNTER, "Number of queries in this database that have been canceled due to pinned buffers", nil, nil},
 		"confl_deadlock":   {COUNTER, "Number of queries in this database that have been canceled due to deadlocks", nil, nil},
 	},
-	"pg_locks": map[string]ColumnMapping{
+	"pg_locks": {
 		"datname": {LABEL, "Name of this database", nil, nil},
 		"mode":    {LABEL, "Type of Lock", nil, nil},
 		"count":   {GAUGE, "Number of locks", nil, nil},
 	},
-	"pg_stat_replication": map[string]ColumnMapping{
+	"pg_stat_replication": {
 		"procpid":          {DISCARD, "Process ID of a WAL sender process", nil, semver.MustParseRange("<9.2.0")},
 		"pid":              {DISCARD, "Process ID of a WAL sender process", nil, semver.MustParseRange(">=9.2.0")},
 		"usesysid":         {DISCARD, "OID of the user logged into this WAL sender process", nil, nil},
@@ -261,7 +240,7 @@ var metricMaps = map[string]map[string]ColumnMapping{
 		"pg_current_xlog_location": {DISCARD, "pg_current_xlog_location", nil, nil},
 		"pg_xlog_location_diff":    {GAUGE, "Lag in bytes between master and slave", nil, semver.MustParseRange(">=9.2.0")},
 	},
-	"pg_stat_activity": map[string]ColumnMapping{
+	"pg_stat_activity": {
 		"datname":         {LABEL, "Name of this database", nil, nil},
 		"state":           {LABEL, "connection state", nil, semver.MustParseRange(">=9.2.0")},
 		"count":           {GAUGE, "number of connections in this state", nil, nil},
@@ -281,7 +260,7 @@ type OverrideQuery struct {
 // Overriding queries for namespaces above.
 // TODO: validate this is a closed set in tests, and there are no overlaps
 var queryOverrides = map[string][]OverrideQuery{
-	"pg_locks": []OverrideQuery{
+	"pg_locks": {
 		{
 			semver.MustParseRange(">0.0.0"),
 			`SELECT pg_database.datname,tmp.mode,COALESCE(count,0) as count
@@ -305,7 +284,7 @@ var queryOverrides = map[string][]OverrideQuery{
 		},
 	},
 
-	"pg_stat_replication": []OverrideQuery{
+	"pg_stat_replication": {
 		{
 			semver.MustParseRange(">=9.2.0"),
 			`
@@ -325,7 +304,7 @@ var queryOverrides = map[string][]OverrideQuery{
 		},
 	},
 
-	"pg_stat_activity": []OverrideQuery{
+	"pg_stat_activity": {
 		// This query only works
 		{
 			semver.MustParseRange(">=9.2.0"),
@@ -691,8 +670,6 @@ type Exporter struct {
 	// Last version used to calculate metric map. If mismatch on scrape,
 	// then maps are recalculated.
 	lastMapVersion semver.Version
-	// Currently active variable map
-	variableMap map[string]MetricMapNamespace
 	// Currently active metric map
 	metricMap map[string]MetricMapNamespace
 	// Currently active query overrides
@@ -723,7 +700,6 @@ func NewExporter(dsn string, userQueriesPath string) *Exporter {
 			Name:      "last_scrape_error",
 			Help:      "Whether the last scrape of metrics from PostgreSQL resulted in an error (1 for error, 0 for success).",
 		}),
-		variableMap:    nil,
 		metricMap:      nil,
 		queryOverrides: nil,
 	}
@@ -773,42 +749,6 @@ func newDesc(subsystem, name, help string) *prometheus.Desc {
 	)
 }
 
-// Query the SHOW variables from the query map
-// TODO: make this more functional
-func queryShowVariables(ch chan<- prometheus.Metric, db *sql.DB, variableMap map[string]MetricMapNamespace) []error {
-	log.Debugln("Querying SHOW variables")
-	nonFatalErrors := []error{}
-
-	for _, mapping := range variableMap {
-		for columnName, columnMapping := range mapping.columnMappings {
-			// Check for a discard request on this value
-			if columnMapping.discard {
-				continue
-			}
-
-			// Use SHOW to get the value
-			row := db.QueryRow(fmt.Sprintf("SHOW %s;", columnName))
-
-			var val interface{}
-			err := row.Scan(&val)
-			if err != nil {
-				nonFatalErrors = append(nonFatalErrors, errors.New(fmt.Sprintln("Error scanning runtime variable:", columnName, err)))
-				continue
-			}
-
-			fval, ok := columnMapping.conversion(val)
-			if !ok {
-				nonFatalErrors = append(nonFatalErrors, errors.New(fmt.Sprintln("Unexpected error parsing column: ", namespace, columnName, val)))
-				continue
-			}
-
-			ch <- prometheus.MustNewConstMetric(columnMapping.desc, columnMapping.vtype, fval)
-		}
-	}
-
-	return nonFatalErrors
-}
-
 // Query within a namespace mapping and emit metrics. Returns fatal errors if
 // the scrape fails, and a slice of errors if they were non-fatal.
 func queryNamespaceMapping(ch chan<- prometheus.Metric, db *sql.DB, namespace string, mapping MetricMapNamespace, queryOverrides map[string]string) ([]error, error) {
@@ -939,11 +879,10 @@ func (e *Exporter) checkMapVersions(ch chan<- prometheus.Metric, db *sql.DB) err
 	semanticVersion, err := parseVersion(versionString)
 
 	// Check if semantic version changed and recalculate maps if needed.
-	if semanticVersion.NE(e.lastMapVersion) || e.variableMap == nil || e.metricMap == nil {
+	if semanticVersion.NE(e.lastMapVersion) || e.metricMap == nil {
 		log.Infoln("Semantic Version Changed:", e.lastMapVersion.String(), "->", semanticVersion.String())
 		e.mappingMtx.Lock()
 
-		e.variableMap = makeDescMap(semanticVersion, variableMaps)
 		e.metricMap = makeDescMap(semanticVersion, metricMaps)
 		e.queryOverrides = makeQueryOverrideMap(semanticVersion, queryOverrides)
 		e.lastMapVersion = semanticVersion
@@ -966,6 +905,24 @@ func (e *Exporter) checkMapVersions(ch chan<- prometheus.Metric, db *sql.DB) err
 	return nil
 }
 
+func getDB(conn string) (*sql.DB, error) {
+	if db == nil {
+		d, err := sql.Open("postgres", conn)
+		if err != nil {
+			return nil, err
+		}
+		err = d.Ping()
+		if err != nil {
+			return nil, err
+		}
+		d.SetMaxOpenConns(1)
+		d.SetMaxIdleConns(1)
+		db = d
+	}
+
+	return db, nil
+}
+
 func (e *Exporter) scrape(ch chan<- prometheus.Metric) {
 	defer func(begun time.Time) {
 		e.duration.Set(time.Since(begun).Seconds())
@@ -974,26 +931,29 @@ func (e *Exporter) scrape(ch chan<- prometheus.Metric) {
 	e.error.Set(0)
 	e.totalScrapes.Inc()
 
-	db, err := sql.Open("postgres", e.dsn)
+	db, err := getDB(e.dsn)
 	if err != nil {
-		log.Infoln("Error opening connection to database:", err)
+		loggableDsn := "could not parse DATA_SOURCE_NAME"
+		if pDsn, pErr := url.Parse(e.dsn); pErr != nil {
+			pDsn.User = url.UserPassword(pDsn.User.Username(), "xxx")
+			loggableDsn = pDsn.String()
+		}
+		log.Infof("Error opening connection to database (%s): %s", loggableDsn, err)
 		e.error.Set(1)
 		return
 	}
-	defer db.Close()
 
 	// Check if map versions need to be updated
 	if err := e.checkMapVersions(ch, db); err != nil {
-		log.Warnln("Postgres version could not be determined. Proceeding with outdated query maps.")
+		log.Warnln("Proceeding with outdated query maps, as the Postgres version could not be determined:", err)
 		e.error.Set(1)
 	}
 
 	// Lock the exporter maps
 	e.mappingMtx.RLock()
 	defer e.mappingMtx.RUnlock()
-	// Handle querying the show variables
-	nonFatalErrors := queryShowVariables(ch, db, e.variableMap)
-	if len(nonFatalErrors) > 0 {
+	if err := querySettings(ch, db); err != nil {
+		log.Infof("Error retrieving settings: %s", err)
 		e.error.Set(1)
 	}
 
@@ -1026,4 +986,7 @@ func main() {
 
 	log.Infof("Starting Server: %s", *listenAddress)
 	log.Fatal(http.ListenAndServe(*listenAddress, nil))
+	if db != nil {
+		defer db.Close()
+	}
 }
